#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new textured image framed (using G'MIC filters)

# (c) Dinasset 2023
#
#   History:
#
#   v.1.1.0: 2023-10: Initial version
#            2023-10-09: fixed afre_localcontrast random error
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
maxwh=7200

############################################################################
# Service routines
#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight, prepstep):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        if realWidthHeight<=maxwh:
            widthheight = realWidthHeight
        else:
            widthheight = maxwh  # max
            gimp.message ("forced max width+height")
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size ------------------------------------------------
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
    
    # create base for effect
    pdb.gimp_image_set_active_layer (newImage,  newBG)

    # optional user choice for preprocessing
    if prepstep>0:  #user did a selection
        #               #"DCP Dehaze (good for light image",
        #               #"Dynamic Range Increase",
        #               #"Equalize Local Histogram",
        #               #"Local Contrast Enhancement",
        #               #"Local Normalization",
        #               #"Local Processing (Equalize)",
        #               #"Local Processing (Normalize)",
        if prepstep==1: # jeje_dehaze 5,1,0.2,1,0,0,0,0,0
            parms='5,1,0.2,1,0,0,0,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -jeje_dehaze "+str(parms)+" ")
        elif prepstep==2: # fx_tk_dri 0.5,0.5,0.5,2.5,0.5,1,1,0
            parms='0.5,0.5,0.5,2.5,0.5,1,1,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_tk_dri "+str(parms)+" ")
        elif prepstep==3: # fx_equalize_local_histograms 75,2,4,100,8,1,16,0,50,50
            parms='75,2,4,100,8,1,16,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_equalize_local_histograms "+str(parms)+" ")
        elif prepstep==4: # fx_LCE 80,0.5,1,1,0,0
            parms='80,0.5,1,1,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_LCE "+str(parms)+" ")
        elif prepstep==5: # fx_normalize_local 2,6,5,20,1,11,0,50,50
            parms='2,6,5,20,1,11,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_normalize_local "+str(parms)+" ")
        elif prepstep==6: # fx_local_processing 1,75,10,50,20,0,7,0,50,50
            parms='1,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
        elif prepstep==7: # fx_local_processing 0,75,10,50,20,0,7,0,50,50
            parms='0,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
    newBG = pdb.gimp_image_get_active_layer (newImage)
            
    # adjust mean value of the base
    # "normal" mean value should be betweeb 117.5 and 137.5
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #gimp.message ("mean="+str(mean))
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    newBG.opacity = 100.0
    
    return(newImage)
#--------------------------------------------------------------------------- 
    
############################################################################
# Service routine (by MareroQ) for Frame colour
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	     
############################################################################
# Main
#---------------------------------------------------------------------------

def PatternFramed (inImage, inLayer, widthheight,
                    frame, size_fr, color_frame,
                    style,
                    regaincolours, prepstep,
                    doFlatten):
      
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    # auto mode and opacity
    patternmode=0			#fixed auto
    patternopacity=0		#fixed auto
    	
    newImage = createOutimage (inImage, inLayer, widthheight, prepstep)
    newBGo = newImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen 
    baseforeffect = newBGo.copy()
    newImage.add_layer (baseforeffect, 0)
    baseforeffect.name = "base for effect"
    imgW = baseforeffect.width
    imgH = baseforeffect.height
    imgsize = imgW+imgH
    frameDX = imgW*size_fr/200
    frameDY = imgH*size_fr/200
    
    #pdb.gimp_message ("working on an image of width+height="+str(imgsize))
    
    # optional user choice for preprocessing
    if prepstep>0:  #user did a selection
        #               #"DCP Dehaze (good for light image",
        #               #"Dynamic Range Increase",
        #               #"Equalize Local Histogram",
        #               #"Local Contrast Enhancement",
        #               #"Local Normalization",
        #               #"Local Processing (Equalize)",
        #               #"Local Processing (Normalize)",
        if prepstep==1: # jeje_dehaze 5,1,0.2,1,0,0,0,0,0
            parms='5,1,0.2,1,0,0,0,0,0'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -jeje_dehaze "+str(parms)+" ")
        elif prepstep==2: # fx_tk_dri 0.5,0.5,0.5,2.5,0.5,1,1,0
            parms='0.5,0.5,0.5,2.5,0.5,1,1,0'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_tk_dri "+str(parms)+" ")
        elif prepstep==3: # fx_equalize_local_histograms 75,2,4,100,8,1,16,0,50,50
            parms='75,2,4,100,8,1,16,0,50,50'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_equalize_local_histograms "+str(parms)+" ")
        elif prepstep==4: # fx_LCE 80,0.5,1,1,0,0
            parms='80,0.5,1,1,0,0'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_LCE "+str(parms)+" ")
        elif prepstep==5: # fx_normalize_local 2,6,5,20,1,11,0,50,50
            parms='2,6,5,20,1,11,0,50,50'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_normalize_local "+str(parms)+" ")
        elif prepstep==6: # fx_local_processing 1,75,10,50,20,0,7,0,50,50
            parms='1,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
        elif prepstep==7: # fx_local_processing 0,75,10,50,20,0,7,0,50,50
            parms='0,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
    baseforeffect = pdb.gimp_image_get_active_layer (newImage)
            
    # check and -if necessary- adjust mean
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect, HISTOGRAM_VALUE,
                                                     0.0, 1.0)
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
      
    #pdb.gimp_display_new (newImage)
    #return
    
    # add local contrast as a preliminary step
    # afre_localcontrast 1,50 # was afre_localcontrast 2,100 >random errors 9Oct
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,0,
        "-v - -afre_localcontrast 1,50 ") 
      
    #pdb.gimp_display_new (newImage)
    #return
    
    # set defaults mode/opacity
    autolayermode = LAYER_MODE_SOFTLIGHT  # default mode
    autolayeropacity = 75.0  # default opacity
    
    # apply chosen GMIC filter: Box_Fitting effect
    if style==0:  #boxfitting
        # fx_boxfitting 4,0,0.25,2,0
        parm1=str(max(1,int(imgsize/1000)))
        parms=parm1+',0,0.25,2,0'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_boxfitting "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Box Fitting "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if patternopacity<1: autolayeropacity = 50.0
                                         
    #pdb.gimp_display_new (newImage)
    #return
         
    if style==1:    #Camouflage
        # fx_camouflage 8,10,100,50,50,50,100,100,100,150,150,150,200,200,200
        parm1=str(max(2,int(imgsize/300)))
        parms=parm1+',10,100,50,50,50,100,100,100,150,150,150,200,200,200' #chg
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_camouflage "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Camouflage "+str(parms)
        Temp = baseforeffect.copy()
        newImage.add_layer (Temp, 0)
        pdb.plug_in_plasma (newImage, Temp, -1, 5.0)
        Temp.mode = LAYER_MODE_LCH_CHROMA
        PatternLayer = pdb.gimp_image_merge_down (newImage, Temp,
        										   CLIP_TO_BOTTOM_LAYER) 
        if patternmode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if patternopacity<1: autolayeropacity = 50.0
              
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==2:    #Canvas
        # fx_canvas 70,45,400,1,70,135,400,0,50,50
        parms='70,45,400,1,70,135,400,0,50,50'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_canvas "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Canvas "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if patternopacity<1: autolayeropacity = 95.0
              
              
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==3:    #Texturize_Canvas
        # texturize_canvas 256,20,1
        parms='256,20,1'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -texturize_canvas "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Texturize_Canvas "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if patternopacity<1: autolayeropacity = 95.0
              
    #pdb.gimp_display_new (newImage)  
    #return
        
    elif style==4:    #Cracks
        # fx_cracks 30,1,255,255,255,128,0,0,50,50
        parms='30,1,255,255,255,128,0,0,50,50'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_cracks "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Cracks "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_LUMA_LIGHTEN_ONLY
        if patternopacity<1: autolayeropacity = 80.0
           
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==5:    #Crystal
        # fx_crystal fx_crystal 100,0,20,0,52.8409,69.5946      #chg
        parms='100,0,20,0,52.8409,69.5946'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)
        
        #there is a need of a proprocess step: add cracks
        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_cracks 30,1,255,255,255,128,0,0,50,50 ")   #std parms  
        # slow, better to scale down before applying
        pdb.gimp_layer_scale (PatternLayer, imgW/2, imgH/2, True)
        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_crystal "+str(parms)+" ")
        pdb.gimp_layer_scale (PatternLayer, imgW, imgH, True)
        
        PatternLayer.name = "Pattern: Crystal "+str(parms)
        pdb.gimp_drawable_brightness_contrast (PatternLayer, 0.5, 0.5)
         
        if patternmode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if patternopacity<1: autolayeropacity = 95.0
           
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==6:    #Crystal_BG
        # fx_crystal_background 10,40,0,100,1
        parms='10,40,0,100,1'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_crystal_background "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Crystal BG "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if patternopacity<1: autolayeropacity = 50.0
           
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==7:    #Denim_samj
        #Denim_samj 5,2,0,40,40,25,50,0,43,108,126,255
        parms='5,2,0,40,40,25,50,0,43,108,126,255'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -Denim_samj "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Denim_samj "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_LUMINANCE
        if patternopacity<1: autolayeropacity = 65.0
           
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==8:    #Halftone
        # fx_halftone 0,0,0,0,5,8,8,5,0.1,0,50,50  
        parms='0,0,0,0,5,8,8,5,0.1,0,50,50'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_halftone "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Halftone "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_LUMINANCE
        if patternopacity<1: autolayeropacity = 85.0
           
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==9:    #Generic_Halftone
        # fx_generic_halftone 0,1,100,10,1,0,3,0,90,1,0,"0,1,100,10,1,0,3,0,90,1,0"
        parms='0,1,100,10,1,0,3,0,90,1,0,"0,1,100,10,1,0,3,0,90,1,0"'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_generic_halftone "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Generic_Halftone "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_VIVID_LIGHT
        if patternopacity<1: autolayeropacity = 50.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==10:    #Halftone_shapes
        # iain_halftone_shapes 12.25,0,0,1,0,0,0
        parms='12.25,0,0,1,0,0,0'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -iain_halftone_shapes "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Halftone_shapes "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_MULTIPLY
        if patternopacity<1: autolayeropacity = 75.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==11:    #Hedcut
        # hedcut 0.5,0.5,0.5,0,0.5,0,1,0
        parms='0.5,0.5,0.5,0,0.5,0,1,0'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -hedcut "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Hedcut "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_MULTIPLY
        if patternopacity<1: autolayeropacity = 75.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==12:    #Lava
        # fx_lava 8,5,3,0,0,50,50 
        parms='8,5,3,0,0,50,50'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_lava "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Lava "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if patternopacity<1: autolayeropacity = 65.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==13:    #Mineral_Mosaic
        # fx_mineral_mosaic 3,32,10,0,0  
        parms='3,32,10,0,0'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_mineral_mosaic "+str(parms)+" ")     
        
        PatternLayer.name = "Pattern: Mineral_Mosaic "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if patternopacity<1: autolayeropacity = 95.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==14:    #Mosaic
        # fx_mosaic 40,0,0,50,50
        parms='40,0,0,50,50'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)

        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_mosaic "+str(parms)+" ")
        PatternLayer.name = "Pattern: Mosaic "+str(parms)
        # evidence of tiles
        Temp = PatternLayer.copy()
        newImage.add_layer (Temp, 0)
        pdb.plug_in_gmic_qt(newImage,Temp, 1,0,
        "-v - -fx_gradient_norm 0,0.015,0,100,1,0,50,50 ")
        Temp.mode = LAYER_MODE_MULTIPLY
        PatternLayer = pdb.gimp_image_merge_down (newImage, Temp,
         										   CLIP_TO_BOTTOM_LAYER)
         
        if patternmode==0: autolayermode = LAYER_MODE_LUMINANCE
        if patternopacity<1: autolayeropacity = 85.0
           
    #pdb.gimp_display_new (newImage)
    #return
      
    elif style==15:    #Shapes (Op Art)
        # fx_shapes 1,16,10,1,5,90,0,0,1,1,0
        parms='1,16,10,1,5,90,0,0,1,1,0'
        PatternLayer = baseforeffect.copy()
        newImage.add_layer (PatternLayer, 0)
         
        pdb.plug_in_gmic_qt(newImage,PatternLayer, 1,0,
        "-v - -fx_shapes "+str(parms)+" ")
        # reset active layer (lost???)
        PatternLayer = pdb.gimp_image_get_active_layer (newImage)
        PatternLayer.name = "Pattern: Shapes "+str(parms)
         
        if patternmode==0: autolayermode = LAYER_MODE_LUMINANCE
        if patternopacity<1: autolayeropacity = 50.0
           
    #pdb.gimp_display_new (newImage)
    #return
    
    else:
        donothing = "do_nothing"
                                                                        
    # COMMON ******************************************************
    PatternLayer = pdb.gimp_image_get_active_layer (newImage)
    
    if patternmode==0: PatternLayer.mode = autolayermode
    if patternopacity<1: PatternLayer.opacity = autolayeropacity
           
    #pdb.gimp_display_new (newImage)
    #return
                                                                        
    final_layer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                   "Pre_Outcome")
    newImage.add_layer (final_layer, 0)
    final_layer.name = "Preliminary Outcome"
    if regaincolours==True:
        final_layer.mode = LAYER_MODE_OVERLAY
        pdb.gimp_item_set_visible (PatternLayer, False)         
    else:    
        final_layer.mode = LAYER_MODE_NORMAL
       
    #pdb.gimp_display_new (newImage)
    #return
     
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (outcome, 0)
    outcome.name = "Outcome (unframed)"
        
    #pdb.gimp_display_new (newImage)
    #return
        
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)
    SH = int(size_fr)
    SV = int(size_fr)
    Round=["0.0","1.0","0.2","0.15","0.1"]	
    Rnd=str(Round[frame])
    XSh = min(10,int(size_fr/6))
    YSh = min(10,int(size_fr/6))
    OuS = min(10,int(size_fr/3))	
    parms=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,'
    parms=parms+str(OuS)+','+str(hex)+','+str(XSh)+','+str(YSh)+',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(newImage, outcome, 1,2,
        "-v - -fx_frame_blur "+str(parms)+" ")
    Framed = pdb.gimp_image_get_active_layer(newImage)
    Framed.mode = LAYER_MODE_NORMAL
         
    #pdb.gimp_display_new (newImage)
     
    stylename = ["Box_Fitting","Camouflage","Canvas","Texturize_Canvas",
                 "Cracks","Crystal","Crystal_BG","Denim_samj",
                 "Halftone","Generic_Halftone","Halftone_shapes","Hedcut",
                 "Lava","Mineral_Mosaic","Mosaic","Shapes"
                 ]     
    Framed.name = st+oname+" with a framed patterned effect, style "
    Framed.name = Framed.name+str(stylename[style])
    # optional user choice for preprocessing
    prepsname = ["",    
                "DCP Dehaze (good for light image",
                "Dynamic Range Increase",
                "Equalize Local Histogram",
                "Local Contrast Enhancement",
                "Local Normalization",
                "Local Processing (Equalize)",
                "Local Processing (Normalize)",
                ]
    
    if prepstep>0:
        Framed.name = Framed.name+" (prep.step choosen="+str(prepsname[prepstep])+"."
                		
    frameDX = imgW*SH/200
    frameDY = imgH*SV/200
    delta = max(frameDX, frameDY)
    bsize = max(imgW, imgH)
    corner=[0.0,100.0,40.0,30.0,20.0]	
    angle=(float(corner[frame]))
    #pdb.gimp_message ("angle="+str(angle))
     	
    # create a frame only layer
    # note that the "clear" is approximate, because it seems that
    #   	GMIC and GIMP calculate differently the curves
    #       in the vertices of a round rectangle
    FrameOnly = Framed.copy()
    newImage.add_layer (FrameOnly, 1)
    FrameOnly.name = "Frame Only"
    pdb.gimp_layer_add_alpha (FrameOnly)
    if frame==0:
        pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
        delta, delta, imgW, imgH)
    elif frame==1:
        pdb.gimp_image_select_ellipse (newImage, CHANNEL_OP_REPLACE,
        delta, delta, imgW, imgH) 	
    elif frame>1:	
        pdb.gimp_image_select_round_rectangle (newImage, CHANNEL_OP_REPLACE,
        delta, delta, imgW, imgH, angle*imgW/100, angle*imgH/100)  	
    pdb.gimp_drawable_edit_clear (FrameOnly)
    pdb.gimp_selection_none (newImage)	
    pdb.gimp_item_set_visible (Framed, False)    
         		                		
    # align all layers to Framed internal image
    # replaced by MrQ solution	
    for layer in newImage.layers:		
    	pdb.gimp_layer_set_offsets(layer,
    	   (pdb.gimp_image_width(newImage)-pdb.gimp_drawable_width(layer))/2,
    	   (pdb.gimp_image_height(newImage)-pdb.gimp_drawable_height(layer))/2)
        
 
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, FrameOnly)
        pdb.gimp_image_remove_layer (newImage, outcome)
        pdb.gimp_image_remove_layer (newImage, final_layer)
        pdb.gimp_image_remove_layer (newImage, PatternLayer)
        pdb.gimp_image_remove_layer (newImage, baseforeffect)
        pdb.gimp_image_remove_layer (newImage, newBGo)
        pdb.gimp_item_set_visible (Framed, True)
        pdb.gimp_image_set_active_layer (newImage, Framed)

    	
    pdb.gimp_display_new (newImage)
    return
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Pattern Framed"

register(
    "PatternFramed",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Pattern Framed...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
         "\n (0=same as input)", 3600, (0, 7200, 100)),
        (PF_OPTION, "frame", "Choose frame format", 0,
                     ["square","round","0.2 rounded square",
                      "0.15 rounded square","0.1 rounded square"
                      ]),
        (PF_SPINNER, "size_fr",    "Frame thickness", 20, (0, 50, 2)),					  
        (PF_COLOR,   "color_frame", "Color frame", (200,200,200)),					  
        (PF_OPTION, "baseeffect", "Choose GMIC base effect ", 0,
                     ["Box_Fitting","Camouflage","Canvas","Texturize_Canvas",
                      "Cracks","Crystal","Crystal_BG","Denim_samj",
                      "Halftone","Generic_Halftone","Halftone_shapes","Hedcut",
                      "Lava","Mineral_Mosaic","Mosaic","Shapes"
                      ]),
        (PF_TOGGLE, "getcolours", "Regain colours for final outcome?", False),
        (PF_OPTION, "preproc", "Choose preprocessing step ", 0,
                     ["*none",
                      "DCP Dehaze (good for light images)",
                      "Dynamic Range Increase",
                      "Equalize Local Histogram",
                      "Local Contrast Enhancement",
                      "Local Normalization",
                      "Local Processing (Equalize)",
                      "Local Processing (Normalize)",
                      ]),        
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    PatternFramed,
    menu=menu
)

main()
